@@ -23,15 +23,15 @@
 #include "setrans_internal.h"
 
 #ifndef DISABLE_SETRANS
-static int mls_enabled = -1;
+static unsigned char has_setrans;
 
 // Simple cache
-static __thread security_context_t prev_t2r_trans = NULL;
-static __thread security_context_t prev_t2r_raw = NULL;
-static __thread security_context_t prev_r2t_trans = NULL;
-static __thread security_context_t prev_r2t_raw = NULL;
+static __thread char * prev_t2r_trans = NULL;
+static __thread char * prev_t2r_raw = NULL;
+static __thread char * prev_r2t_trans = NULL;
+static __thread char * prev_r2t_raw = NULL;
 static __thread char *prev_r2c_trans = NULL;
-static __thread security_context_t prev_r2c_raw = NULL;
+static __thread char * prev_r2c_raw = NULL;
 
 static pthread_once_t once = PTHREAD_ONCE_INIT;
 static pthread_key_t destructor_key;
@@ -56,7 +56,10 @@ static int setransd_open(void)
 	{
 		fd = socket(PF_UNIX, SOCK_STREAM, 0);
 		if (fd >= 0)
-			fcntl(fd, F_SETFD, FD_CLOEXEC);
+			if (fcntl(fd, F_SETFD, FD_CLOEXEC)) {
+				close(fd);
+				return -1;
+			}
 	}
 	if (fd < 0)
 		return -1;
@@ -151,9 +154,10 @@ receive_response(int fd, uint32_t function, char **outdata, int32_t * ret_val)
 	}
 
 	data = malloc(data_size);
-	if (!data) {
+	if (!data)
 		return -1;
-	}
+	/* coveriety doesn't realize that data will be initialized in readv */
+	memset(data, 0, data_size);
 
 	resp_data.iov_base = data;
 	resp_data.iov_len = data_size;
@@ -253,14 +257,20 @@ static void setrans_thread_destructor(void __attribute__((unused)) *unused)
 	free(prev_r2c_raw);
 }
 
-void __attribute__((destructor)) setrans_lib_destructor(void)
+void __attribute__((destructor)) setrans_lib_destructor(void);
+
+void hidden __attribute__((destructor)) setrans_lib_destructor(void)
 {
+	if (!has_setrans)
+		return;
 	if (destructor_key_initialized)
 		__selinux_key_delete(destructor_key);
 }
 
 static inline void init_thread_destructor(void)
 {
+	if (!has_setrans)
+		return;
 	if (destructor_initialized == 0) {
 		__selinux_setspecific(destructor_key, (void *)1);
 		destructor_initialized = 1;
@@ -269,14 +279,15 @@ static inline void init_thread_destructor(void)
 
 static void init_context_translations(void)
 {
+	has_setrans = (access(SETRANS_UNIX_SOCKET, F_OK) == 0);
+	if (!has_setrans)
+		return;
 	if (__selinux_key_create(&destructor_key, setrans_thread_destructor) == 0)
 		destructor_key_initialized = 1;
-
-	mls_enabled = is_selinux_mls_enabled();
 }
 
-int selinux_trans_to_raw_context(const security_context_t trans,
-				 security_context_t * rawp)
+int selinux_trans_to_raw_context(const char * trans,
+				 char ** rawp)
 {
 	if (!trans) {
 		*rawp = NULL;
@@ -286,7 +297,7 @@ int selinux_trans_to_raw_context(const security_context_t trans,
 	__selinux_once(once, init_context_translations);
 	init_thread_destructor();
 
-	if (!mls_enabled) {
+	if (!has_setrans) {
 		*rawp = strdup(trans);
 		goto out;
 	}
@@ -317,8 +328,8 @@ int selinux_trans_to_raw_context(const security_context_t trans,
 
 hidden_def(selinux_trans_to_raw_context)
 
-int selinux_raw_to_trans_context(const security_context_t raw,
-				 security_context_t * transp)
+int selinux_raw_to_trans_context(const char * raw,
+				 char ** transp)
 {
 	if (!raw) {
 		*transp = NULL;
@@ -328,7 +339,7 @@ int selinux_raw_to_trans_context(const security_context_t raw,
 	__selinux_once(once, init_context_translations);
 	init_thread_destructor();
 
-	if (!mls_enabled) {
+	if (!has_setrans)  {
 		*transp = strdup(raw);
 		goto out;
 	}
@@ -359,7 +370,7 @@ int selinux_raw_to_trans_context(const security_context_t raw,
 
 hidden_def(selinux_raw_to_trans_context)
 
-int selinux_raw_context_to_color(const security_context_t raw, char **transp)
+int selinux_raw_context_to_color(const char * raw, char **transp)
 {
 	if (!raw) {
 		*transp = NULL;
@@ -369,6 +380,11 @@ int selinux_raw_context_to_color(const security_context_t raw, char **transp)
 	__selinux_once(once, init_context_translations);
 	init_thread_destructor();
 
+	if (!has_setrans) {
+		*transp = strdup(raw);
+		goto out;
+	}
+
 	if (prev_r2c_raw && strcmp(prev_r2c_raw, raw) == 0) {
 		*transp = strdup(prev_r2c_trans);
 	} else {
@@ -396,8 +412,8 @@ int selinux_raw_context_to_color(const security_context_t raw, char **transp)
 hidden_def(selinux_raw_context_to_color)
 #else /*DISABLE_SETRANS*/
 
-int selinux_trans_to_raw_context(const security_context_t trans,
-				 security_context_t * rawp)
+int selinux_trans_to_raw_context(const char * trans,
+				 char ** rawp)
 {
 	if (!trans) {
 		*rawp = NULL;
@@ -411,8 +427,8 @@ int selinux_trans_to_raw_context(const security_context_t trans,
 
 hidden_def(selinux_trans_to_raw_context)
 
-int selinux_raw_to_trans_context(const security_context_t raw,
-				 security_context_t * transp)
+int selinux_raw_to_trans_context(const char * raw,
+				 char ** transp)
 {
 	if (!raw) {
 		*transp = NULL;
