@@ -8,7 +8,7 @@
 #include <ctype.h>
 #include <selinux/selinux.h>
 
-int main(int argc, char **argv) 
+int main(int argc, char **argv)
 {
 	char **buf, **ptr;
 	int ret;
@@ -20,7 +20,8 @@ int main(int argc, char **argv)
 
 	ret = security_compute_user(argv[1], argv[2], &buf);
 	if (ret < 0) {
-		fprintf(stderr, "%s:  security_compute_user(%s,%s) failed\n", argv[0], argv[1], argv[2]);
+		fprintf(stderr, "%s:  security_compute_user(%s,%s) failed\n",
+			argv[0], argv[1], argv[2]);
 		exit(2);
 	}
 
